#!/usr/bin/perl -w

# DrawDUDI v1.0.2 - Liest Daten fr ein Durchlaufdiagramm ein und zeichnet es
# Copyright (c) 2001, 2002 Dirk Swiniartzki (pamela@planet-dirk.de)
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License version 2
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#    -----------------------------------------------------------
#    Dieses Programm ist freie Software. Sie knnen es unter
#    den Bedingungen der GNU General Public License Version 2,
#    wie von der Free Software Foundation herausgegeben, weitergeben
#    und/oder modifizieren.
#
#    Die Verffentlichung dieses Programms erfolgt in der
#    Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
#    GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
#    der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
#    Details finden Sie in der GNU General Public License.
#
#    Sie sollten eine Kopie der GNU General Public License zusammen
#    mit diesem Programm erhalten haben. Falls nicht, schreiben Sie
#    an die Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#    MA 02139, USA.
#    ----------------------------------------------------------------


use strict;
use GD;             # GD-Modul, zu installieren
use File::Basename; # Standardmodul

my @files;    # Liste der Dateien
my @config;   # Konfigurationsdaten
my ($a, $b);  # temporre Variable

# ---------- init ----------
# initialisiert das Programm
# Parameter : -
# Rckgabewerte : @config

sub init {

   my @config;   # Nimmt die Konfigurationsdaten auf
#   my ($a, $b);  # temporre Variablen
   #my @files;    # Liste der zu bearbeitenden Dateien

   # momentan wird die Konfig mit festen Werten gefllt
   @config=(1024, 768, 100, 100, 100, 100, 3, 3, 1, $ARGV[0]);
   # Konfig : Breite, Hhe, Rand links, rechts, oben, unten, Schriftgre (1-5), Linienbreite1, Linienbreite2, Verzeichnis

   return(@config);
}

# ---------- drawdudi ----------
# liest die Daten fr das DuDi ein und zeichnet es
# Parameter : @config, Dateiname
# Rckgabewerte : keine

sub drawdudi {

   my ($breite, $hoehe, $randl, $randr, $rando, $randu, $schrift, $linie1, $linie2, $dir, $datei)=@_;
   my ($a, $b, $c, $d, $e);# temporre Variablen
   my $bild;               # Handle fr das Bild
   my %color;              # Handles fr die Farben
   my @xwert;              # Die x-Koordinaten der Kurvenpunkte
   my @ywert1;             # Die y-Koordinaten der Kurvenpunkte - Zugang
   my @ywert2;             # Die y-Koordinaten der Kurvenpunkte - Abgang
   my @ywert3;             # Die y-Koordinaten der Kurvenpunkte - Bestand
   my ($nullx, $nully);    # x- und y-Wert des Ursprungs des Koordinatensystems - nur eine Erleichterung
   my $xscale;             # Proportionalittsfaktor fr x-Achse
   my ($yscale1, $yscale2);# Proportionalittsfaktor fr y-Achse, $yscale2 noch unbenutzt, evtl. fr extra-Bestandslinie
   my @data;               # nimmt zu verarbeitenden Datensatz auf
   my ($tiefe, $weite);    # Hhe und Breite der Diagrammflche
   my ($altx, $alty1);     # Nimmt die voherigen x- und y-Werte auf
   my ($alty2, $alty3);    # dito, korrespondierend mit @ywert[1-3]
   my $arbeitssys;         # Name des Arbeitssystems

   # Kennwerte einlesen
   print "Beginne mit Einlesen der Daten aus Datei $datei \n";
   open(DATEN, $datei) || die("Konnte Datei $datei nicht ffnen : $!\n");
   $a=<DATEN>;
   chomp($a);
   while ($a=~/ /) {$a=~s/ //;}
   ($b, $arbeitssys)=split(/:/, $a);

   while (<DATEN>) { # Suche Beginn des Untersuchungszeitraumes
      chomp;
      last if (/^--- Beginn/);}

   while (<DATEN>) { # Lese jetzt die Werte ein
      chomp;
      last if (/^--- Ende/);
      @data=split(/;/, $_); #/
      push(@xwert, $data[0]);  # Tag Nr.
      push(@ywert1, $data[3]); # Zugang kumuliert
      push(@ywert2, $data[4]); # Abgang kumuliert
      push(@ywert3, $data[5]);}# Bestand

   close(DATEN);

   # Jetzt einige Koordinaten und Faktoren berechnen
   $nullx=$randl;
   $nully=$hoehe-$randu;
   $tiefe=$hoehe-$rando-$randu;
   $weite=$breite-$randl-$randr;
   $xscale=$weite/($xwert[-1]-$xwert[0]); #/
   $yscale1=$tiefe/$ywert1[-1]; #/      # y-Werte +10%, damit etwas Abstand

   # Jetzt das Bild vorbereiten
   print "Zeichne Durchlaufdiagramm fr Arbeitssystem $arbeitssys.\n";
   $bild=GD::Image->new($breite, $hoehe);
   $color{"weiss"}  = $bild->colorAllocate(255,255,255); # wei ist automatisch Hintergrundfarbe
   $color{"schwarz"}= $bild->colorAllocate(0,0,0);
   $color{"blau"}   = $bild->colorAllocate(0,0,200);
   $color{"rot"}    = $bild->colorAllocate(200,0,0);
   $color{"gruen"}  = $bild->colorAllocate(0,200,0);
   $color{"gelb"}   = $bild->colorAllocate(0,150,150);

   # Achsen und Skalierung zeichnen
   $bild->setStyle($color{"schwarz"}, $color{"schwarz"}, $color{"weiss"}, $color{"weiss"});
   for ($a=0; $a<11; $a++) { # Skalen zeichnen, je 10 Teilstriche
      $b=int($tiefe*($a/10)); #/# die Anzahl Pixel fr y-Achse
      $c=int($weite*($a/10)); #/# die Anzahl Pixel fr x-Achse
      $bild->line($nullx+$c, $nully+1, $nullx+$c, $nully+10, $color{"schwarz"});  # x-Achse
      $bild->line($nullx-1, $nully-$b, $nullx-10, $nully-$b, $color{"schwarz"});  # y1-Achse
      #$bild->line($nullx+$weite+1, $nully-$c, $nullx+$weite+10, $nully-$c, $color{"schwarz"}); # y2-Achse
      $bild->line($nullx, $nully-$b, $nullx+$weite-1, $nully-$b, gdStyled);} # gestrichelte Linie in Diagrammflche

   for ($a=1; $a<$linie1+1; $a++) {  # Achsen zeichnen
      $bild->line($nullx, $nully+$a, $nullx+$weite, $nully+$a, $color{"schwarz"});  # x-Achse
      $bild->line($nullx-$a, $rando, $nullx-$a, $nully+$linie1, $color{"schwarz"});}# y1-Achse
      #$bild->line($nullx+$weite+$a, $rando, $nullx+$weite+$a, $nully+$linie1, $color{"schwarz"});} # y2-Achse

   # Achsenbeschriftung
   $bild->stringUp(gdSmallFont, int($randl/2)-12, int($hoehe/2)-30 ,"Arbeit [h]", $color{"blau"}); # y1-Achse, Arbeit, links
   #$bild->stringUp(gdSmallFont, $breite-int($randr/2), int($hoehe/2)-30 ,"Reichweite", $color{"rot"}); # y2-Achse, Reichweite, rechts
   $bild->string(gdSmallFont, int($breite/2)-9, $hoehe-int($randu/2), "Tag", $color{"schwarz"}); # x-Achse, unten

   # Diagrammbeschriftung
   $bild->string(gdLargeFont, int($breite/4), int($rando/2)-8, "Durchlaufdiagramm Arbeitssystem $arbeitssys", $color{"schwarz"});

   # Skalenbeschriftung
   for ($a=0; $a<11; $a++) {
      $b=int($tiefe*($a/10)); #/ # die Anzahl Pixel fr y-Achse, durch 11 teilen, da oben yscale*1.1
      $c=int($weite*($a/10)); #/ # die Anzahl Pixel fr x-Achse
      $d=$xwert[0]+int((($xwert[-1]-$xwert[0])/10)*$a);  #/
      $bild->string(gdSmallFont, $randl+int($c)-6, $hoehe-$randu+15, "$d", $color{"schwarz"});
      $d=int(($ywert1[-1]/10)*$a); #/
      $bild->stringUp(gdSmallFont, $randl-27, $hoehe-$randu-int($b)+6, "$d", $color{"schwarz"});}
      #$d=int($yscale2*$b);
      #$bild->stringUp(gdSmallFont, $breite-$randr+15, $hoehe-$randu-int($b*$a)+6, "$d", $color{"schwarz"});}

   # Kurven zeichnen vorbereiten
   $altx=$nullx;   # $altx, $alty1, $alty2 nehmen den letzten Eckpunkt auf
   $alty1=$ywert1[0];  # Maeinheit ist Pixel fr $altx, Arbeit fr $alty1, Pixel fr $alty2 !
   $alty2=$nully-int($alty1*$yscale1);

   # Nun die Zugangskurve zeichnen
   for ($a=1; $a<(@xwert-1); $a++) {

      next if ($ywert1[$a]==$alty1);
      $b=$nullx+int($a*$xscale);         # Der neue x-Wert, bis zu dem die Linie gezogen werden mu
      $c=$linie1-int($linie1/2)-1; #/ # Hilfswert fr die korrekte Linienbreite
      $d=$nully-int($ywert1[$a]*$yscale1);# Der neue y-Wert
      $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"blau"}); #/ # waagerechte Linie
      $bild->filledRectangle($b-int($linie1/2), $d-int($linie1/2), $b+$c, $alty2+$c, $color{"blau"}); # senkrechte Linie
      $altx=$b;
      $alty1=$ywert1[$a];
      $alty2=$d;}

   # Hier folgt noch einmal ein Schleifendurchlauf zum Zeichnen der Zugangslinie
   # Er ist ntig, weil die Linie zum letzten Punkt auf jeden Fall gezeichnet werden
   # mu, damit am Ende nicht ein Stck fehlt, wie es passieren kann, wenn der letzte Wert
   # gleich dem vorletzten ist (das "next if" in der Schleife !)
   $b=$nullx+int($a*$xscale);         # Der neue x-Wert, bis zu dem die Linie gezogen werden mu
   $c=$linie1-int($linie1/2)-1; #/ # Hilfswert fr die korrekte Linienbreite
   $d=$nully-int($ywert1[$a]*$yscale1);# Der neue y-Wert
   $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"blau"}); #/ # waagerechte Linie
   $bild->filledRectangle($b-int($linie1/2), $d-int($linie1/2), $b+$c, $alty2+$c, $color{"blau"}); # senkrechte Linie

   # Nun die Abgangskurve zeichnen
   $altx=$nullx;   # $altx, $alty1, $alty2 nehmen den letzten Eckpunkt auf
   $alty1=$ywert2[0];  # Maeinheit ist Pixel fr $altx, Arbeit fr $alty1, Pixel fr $alty2 !
   $alty2=$nully-int($alty1*$yscale1);

   for ($a=1; $a<(@xwert-1); $a++) {
      next if ($ywert2[$a]==$alty1);
      $b=$nullx+int($a*$xscale);          # Der neue x-Wert, bis zu dem die Linie gezogen werden mu
      $c=$linie1-int($linie1/2)-1; #/ # Hilfswert fr die korrekte Linienbreite
      $d=$nully-int($ywert2[$a]*$yscale1);# Der neue y-Wert
      $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"rot"}); #/ # waagerechte Linie
      $bild->filledRectangle($b-int($linie1/2), $d-int($linie1/2), $b+$c, $alty2+$c, $color{"rot"}); # senkrechte Linie
      $altx=$b;
      $alty1=$ywert2[$a];
      $alty2=$d;}

   $b=$nullx+int($a*$xscale);          # Der neue x-Wert, bis zu dem die Linie gezogen werden mu
   $c=$linie1-int($linie1/2)-1; #/ # Hilfswert fr die korrekte Linienbreite
   $d=$nully-int($ywert2[$a]*$yscale1);# Der neue y-Wert
   $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"rot"}); #/ # waagerechte Linie
   $bild->filledRectangle($b-int($linie1/2), $d-int($linie1/2), $b+$c, $alty2+$c, $color{"rot"}); # senkrechte Linie

   # Zum Schlu noch die Bestandskurve
   $altx=$nullx;   # $altx, $alty1, $alty2 nehmen den letzten Eckpunkt auf
   $alty1=$ywert3[0];  # Maeinheit ist Pixel fr $altx, Arbeit fr $alty1, Pixel fr $alty2 !
   $alty2=$nully-int($alty1*$yscale1);

   for ($a=1; $a<@xwert; $a++) {
      next if ($ywert3[$a]==$alty1);
      $b=$nullx+int($a*$xscale);          # Der neue x-Wert, bis zu dem die Linie gezogen werden mu
      $c=$linie1-int($linie1/2)-1; #/ # Hilfswert fr die korrekte Linienbreite
      $d=$nully-int($ywert3[$a]*$yscale1);# Der neue y-Wert
      if ($ywert3[$a]>$alty1) {  # senkrechte Linie geht hoch
         $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"gruen"}); #/ # waagerechte Linie
         $bild->filledRectangle($b-int($linie1/2), $d-int($linie1/2), $b+$c, $alty2+$c, $color{"gruen"}); # senkrechte Linie
         $altx=$b;
         $alty1=$ywert3[$a];
         $alty2=$d;}
      else { # senkrechte Linie geht herunter
         $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"gruen"}); #/ # waagerechte Linie
         $bild->filledRectangle($b-int($linie1/2), $alty2-int($linie1/2), $b+$c, $d+$c, $color{"gruen"}); # senkrechte Linie
         $altx=$b;
         $alty1=$ywert3[$a];
         $alty2=$d;}}

   # Hier folgt noch einmal ein Schleifendurchlauf zum Zeichnen der Bedarfslinie
   # Er ist ntig, weil die Linie zum letzten Punkt auf jeden Fall gezeichnet werden
   # mu, damit am Ende nicht ein Stck fehlt, wie es passieren kann, wenn der letzte Wert
   # gleich dem vorletzten ist (das "next if" in der Schleife !)
   $b=$nullx+int($a*$xscale);          # Der neue x-Wert, bis zu dem die Linie gezogen werden mu
   $c=$linie1-int($linie1/2)-1; #/ # Hilfswert fr die korrekte Linienbreite
   $d=$nully-int($ywert3[$a]*$yscale1);# Der neue y-Wert
   if ($ywert3[-1]>$alty1) {  # senkrechte Linie geht hoch
      $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"gruen"}); #/ # waagerechte Linie
      $bild->filledRectangle($b-int($linie1/2), $d-int($linie1/2), $b+$c, $alty2+$c, $color{"gruen"});} # senkrechte Linie
   else { # senkrechte Linie geht herunter
      $bild->filledRectangle($altx, $alty2-int($linie1/2), $b, $alty2+$c, $color{"gruen"}); #/ # waagerechte Linie
      $bild->filledRectangle($b-int($linie1/2), $alty2-int($linie1/2), $b+$c, $d+$c, $color{"gruen"});} # senkrechte Linie

   # Ausgabe des Diagramms
   $a=$dir."dudi_$arbeitssys.png";
   print "Speichere Durchlaufdiagramm in Datei $a\n";
   open(OUTPUT, ">$a") || die("Konnte Datei $a nicht ffnen : $!\n");
   binmode OUTPUT;
   print OUTPUT $bild->png;
   close(OUTPUT);
   }

# ---------- Hauptprogramm ----------
print <<ANNOUNCE;
DrawDuDi v1.0.2 - Zeichnet Durchlaufdiagramme
Copyright (C) 2001, 2002 Dirk Swiniartzki pamela\@planet-dirk.de

Dieses Programm ist freie Software. Sie knnen es unter
den Bedingungen der GNU General Public License Version 2,
wie von der Free Software Foundation herausgegeben, weitergeben
und/oder modifizieren.

Die Verffentlichung dieses Programms erfolgt in der
Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
ANNOUNCE

print "\nSyntax : drawdudi.plx <Verzeichnis>\n";

@config=init(); # @config= siehe init();
if (@ARGV>0) {
   $b=$ARGV[0];}
else {
   $b="./";}
$b.="dudi_*.csv";
@files=glob($b); # Dateiliste einlesen

foreach $a (@files) {
   drawdudi(@config, $a);}

__END__


