#!/usr/bin/perl -w

# DrawHisto v1.0.2 - Liest Daten fr ein Histogramm ein und zeichnet es
# Copyright (C) 2001, 2002 Dirk Swiniartzki (pamela@planet-dirk.de)
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License version 2
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#    -----------------------------------------------------------
#    Dieses Programm ist freie Software. Sie knnen es unter
#    den Bedingungen der GNU General Public License Version 2,
#    wie von der Free Software Foundation herausgegeben, weitergeben
#    und/oder modifizieren.
#
#    Die Verffentlichung dieses Programms erfolgt in der
#    Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
#    GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
#    der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
#    Details finden Sie in der GNU General Public License.
#
#    Sie sollten eine Kopie der GNU General Public License zusammen
#    mit diesem Programm erhalten haben. Falls nicht, schreiben Sie
#    an die Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#    MA 02139, USA.
#    ----------------------------------------------------------------

use strict;
use GD;             # GD-Modul, zu installieren
use File::Basename; # Standardmodul

my @files;    # Liste der Dateien
my @config;   # Konfigurationsdaten
my ($a, $b);  # temporre Variable

# ---------- init ----------
# initialisiert das Programm
# Parameter : -
# Rckgabewerte : @config

sub init {

   my @config;   # Nimmt die Konfigurationsdaten auf
#   my ($a, $b);  # temporre Variablen
   my @files;    # Liste der zu bearbeitenden Dateien

   # momentan wird die Konfig mit festen Werten gefllt
   @config=(1024, 768, 100, 100, 100, 100, 3, 3, 1, $ARGV[0]);
   # Konfig : Breite, Hhe, Rand links, rechts, oben, unten, Schriftgre (1-5), Linienbreite1, Linienbreite2, Verzeichnis

   # Jetzt die Dateiliste aufbauen
#   @files=glob("$config[1]pkl_*.csv");

   return(@config);
}

# ---------- drawhisto ----------
# liest die Daten fr das Histogramm ein und zeichnet es
# Parameter : @config, Dateiname
# Rckgabewerte :

sub drawhisto {

   my ($breite, $hoehe, $randl, $randr, $rando, $randu, $schrift, $linie1, $linie2, $dir, $datei)=@_;
   my ($a, $b, $c, $d);    # temporre Variablen
   my $bild;               # Handle fr das Bild
   my ($typ, $max, $arbeitssys); # nehmen die Werte fr Typ (ZAU/ZDL), max. Wert und Bezeichnung Arbeitssystem auf
   my %color;              # Handles fr die Farben
   my @xwert;              # Die Klassenbezeichnung
   my @ywert;              # Die Hufigkeiten der Klassen
   my ($nullx, $nully);    # x- und y-Wert des Ursprungs des Koordinatensystems - nur eine Erleichterung
   my $xscale;             # Proportionalittsfaktor fr x-Achse
   my $yscale;             # Proportionalittsfaktor fr y-Achse
   my @data;               # nimmt zu verarbeitenden Datensatz auf
   my ($tiefe, $weite);    # Hhe und Breite der Diagrammflche
   my ($base, $scales);    # Basisgre und Anz. Skaleneinteilung y-Achse

   # Kennwerte einlesen
   print "Lese Daten aus Datei $datei .\n";
   open(DATEN, $datei) || die("Konnte Datei $datei nicht ffnen : $!\n");
   $_=<DATEN>; # Einleitung einlesen
   chomp;
   @data=split(/ /, $_); # Bezeichnung des Arbeitssystems und des Typs ermitteln
   $arbeitssys=$data[-1];
   $typ=$data[3];

   <DATEN>; # Leerzeile berspringen
   <DATEN>; # Tabellenkopf berspringen
   while (<DATEN>) { # Lese jetzt die Werte ein
      chomp;
      @data=split(/;/, $_); #/
      push(@xwert, $data[0]);
      push(@ywert, $data[1]);}
   close(DATEN);

   foreach $a (@xwert) { # die Klassenbezeichnung noch etwas krzen
      $a=~s/bis ausschlielich /-/;}
   $xwert[-1]=~s/ und hher/+/;

   $max=0;
   foreach $a (@ywert) { # dem Maximalwert bestimmen
      $max=$a if ($max<$a);}

   # Jetzt einige Koordinaten und Faktoren berechnen
   $nullx=$randl;
   $nully=$hoehe-$randu;
   $tiefe=$hoehe-$rando-$randu;
   $weite=$breite-$randl-$randr;
   $xscale=$weite/@xwert; #/
   # $yscale : siehe unten

   # Jetzt das Bild vorbereiten
   print "Zeichne Histogramm fr Arbeitssystem $arbeitssys.\n";
   $bild=GD::Image->new($breite, $hoehe);
   $color{"weiss"}  = $bild->colorAllocate(255,255,255); # wei ist automatisch Hintergrundfarbe
   $color{"schwarz"}= $bild->colorAllocate(0,0,0);
   $color{"blau"}   = $bild->colorAllocate(0,0,200);
   $color{"rot"}    = $bild->colorAllocate(200,0,0);
   $color{"gruen"}  = $bild->colorAllocate(0,200,0);
   $color{"gelb"}   = $bild->colorAllocate(0,150,150);

   # Rahmen zeichnen
   $bild->rectangle($nullx, $rando, $nullx+$weite, $nully, $color{"schwarz"});
   $bild->rectangle($nullx+1, $rando+1, $nullx+$weite-1, $nully-1, $color{"schwarz"});

   # Skalen zeichnen
   for ($a=0; $a<(@xwert+1); $a++) {
      $b=int($weite*($a/@xwert)); # / # die Anzahl Pixel fr x-Achse
      $bild->line($nullx+$b, $nully+1, $nullx+$b, $nully+10, $color{"schwarz"});} # x-Achse

   # automatische Skalierung y-Achse
   $c=10; # Vorgabe sind 10 Klasen
   $b=int($max/$c); # / # Basisgre berechnen fr $c Klassen
   $b++ if ( $b<=($max/$c) ); # /
   if ($b<1) { # Wenn Basisgre<1 (d.h. $max<10), dann Anzahl Klassen anpassen
      $b=1;
      $c=int($max/$b);} # /
   $yscale=$tiefe/($b*$c); #/
   $base=$b;
   $scales=$c;

   # y-Skala zeichnen
   for ($a=0; $a<($scales+1); $a++) {
      $d=int($tiefe*($a/$scales)); #/
      $bild->line($nullx-10, $nully-$d, $nullx+$weite, $nully-$d, $color{"schwarz"});}

   # Achsenbeschriftung
   $bild->stringUp(gdSmallFont, int($randl/2)-12, int($hoehe/2)-30 ,"Hufigkeit", $color{"schwarz"}); # y1-Achse, Leistung, links
   $bild->string(gdSmallFont, int($breite/2)-48, $hoehe-int($randu/2), "Klasse [bis ...]", $color{"schwarz"}); # x-Achse, unten

   # Diagrammbeschriftung
   $bild->string(gdLargeFont, int($breite/4), int($rando/2)-8, "$typ-Histogramm Arbeitssystem $arbeitssys", $color{"schwarz"});

   # Skalenbeschriftung x-Achse
   for ($a=1; $a<(@xwert+1); $a++) {
      $b=int($weite*($a/@xwert)); #/ # die rechte Grenze der Spalte, in die das Rechteck kommt
      $c=int($weite*(($a-1)/@xwert)); #/ # die linke Grenze
      $d=$nullx+int(($b-$c)/2)+$c; #/ # die Mitte
      @data=split(//, $xwert[$a-1]);
      $d-=(@data*3); # den Versatz fr Zentrierung bercksichtigen
      $bild->string(gdSmallFont, $d, $nully+10, $xwert[$a-1], $color{"schwarz"});}

   # Skalenbeschriftung y-Achse
   for ($a=0; $a<($scales+1); $a++) {
      $b=int($tiefe*($a/$scales)); #/
      $c=$a*$base;
      @data=split(//, $c);
      $bild->string(gdSmallFont, $nullx-(@data*6)-15, $nully-$b-6, "$c", $color{"schwarz"});}

   # Nun die Balken zeichnen
   # eventuell mit Skalenbeschriftung x-Achse zusammenfassen
   for ($a=1; $a<(@xwert+1); $a++) {
      $b=int($weite*($a/@xwert)); # /# die rechte Grenze der Spalte, in die das Rechteck kommt
      $c=int($weite*(($a-1)/@xwert)); # /# die linke Grenze
      $d=int(($b-$c)/4); #/
      $b-=$d;
      $c+=$d;
      $d=int($yscale*$ywert[$a-1]);
      $bild->rectangle($nullx+$c, $nully-$d, $nullx+$b, $nully, $color{"schwarz"});
      $bild->filledRectangle($nullx+$c+1, $nully-$d+1, $nullx+$b-1, $nully-1, $color{"rot"});}

   # Ausgabe des Diagramms
   $a=$dir."histo_$typ\_$arbeitssys.png"; # der Backslash mu da hin ! (sonst Fehler)
   print "Speichere Histogramm in Datei $a\n\n";
   open(OUTPUT, ">$a") || die("Konnte Datei $a nicht ffnen : $!\n");
   binmode OUTPUT;
   print OUTPUT $bild->png;
   close(OUTPUT);
   }

# ---------- Hauptprogramm ----------
print <<ANNOUNCE;
DrawHisto v1.0.2 - zeichnet Histogramme der Auftrags - und Durchlaufzeit
Copyright (C) 2001, 2002 Dirk Swiniartzki pamela\@planet-dirk.de

Dieses Programm ist freie Software. Sie knnen es unter
den Bedingungen der GNU General Public License Version 2,
wie von der Free Software Foundation herausgegeben, weitergeben
und/oder modifizieren.

Die Verffentlichung dieses Programms erfolgt in der
Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
ANNOUNCE

print "\nSyntax : drawhisto.plx <Verzeichnis>\n\n";

@config=init(); # @config= siehe init();
if (@ARGV>0) {
   $b=$ARGV[0];}
else {
   $b="./";}

$b.="histo_*.csv";
@files=glob($b); # Dateiliste einlesen

foreach $a (@files) {
   drawhisto(@config, $a);}

__END__


