#!/usr/bin/perl -w

# DrawTA v1.0.2 - Liest Daten zur Terminabweichung ein und zeichnet Diagramm
# Copyright (C) 2001, 2002 Dirk Swiniartzki (pamela@planet-dirk.de)
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License version 2
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#    -----------------------------------------------------------
#    Dieses Programm ist freie Software. Sie knnen es unter
#    den Bedingungen der GNU General Public License Version 2,
#    wie von der Free Software Foundation herausgegeben, weitergeben
#    und/oder modifizieren.
#
#    Die Verffentlichung dieses Programms erfolgt in der
#    Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
#    GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
#    der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
#    Details finden Sie in der GNU General Public License.
#
#    Sie sollten eine Kopie der GNU General Public License zusammen
#    mit diesem Programm erhalten haben. Falls nicht, schreiben Sie
#    an die Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#    MA 02139, USA.
#    ----------------------------------------------------------------

use strict;
use GD;             # GD-Modul, zu installieren
use File::Basename; # Standardmodul

my $file;     # Terminabweichungsdatei
my @config;   # Konfigurationsdaten
my $a;        # temporre Variable

# ---------- init ----------
# initialisiert das Programm
# Parameter : -
# Rckgabewerte : @config

sub init {

   my @config;   # Nimmt die Konfigurationsdaten auf
#   my ($a, $b);  # temporre Variablen
   my @files;    # Liste der zu bearbeitenden Dateien

   # momentan wird die Konfig mit festen Werten gefllt
   @config=(1024, 768, 100, 100, 100, 100, 3, 3, 1, $ARGV[0]);
   # Konfig : Breite, Hhe, Rand links, rechts, oben, unten, Schriftgre (1-5), Linienbreite1, Linienbreite2, Verzeichnis

   # Jetzt die Dateiliste aufbauen
#   @files=glob("$config[1]pkl_*.csv");

   return(@config);
}

# ---------- drawta ----------
# liest die Daten fr die Terminabweichung ein und zeichnet Diagramm
# Parameter : @config, Dateiname
# Rckgabewerte :

sub drawta {

   my ($breite, $hoehe, $randl, $randr, $rando, $randu, $schrift, $linie1, $linie2, $dir, $datei)=@_;
   my ($a, $b, $c, $d, $e, $f);# temporre Variablen
   my $bild;               # Handle fr das Bild
   my ($bimin, $lmax, $arbeitssys); # nimmt einige Kennwerte fr die PKL auf
   my %color;              # Handles fr die Farben
   my @xwert;              # x-Wert - relative Zugangsterminabweichung
   my @ywert;              # y-Wert - Verzgerung
   my ($nullx, $nully);    # untere linke Ecke der Diagrammflche - nur eine Erleichterung
   my $xscale;             # Proportionalittsfaktor fr x-Achse : rel. TAzugang
   my $yscale;             # Proportionalittsfaktor fr y-Achse : Verzgerung
   my @data;               # nimmt zu verarbeitenden Datensatz auf
   my ($tiefe, $weite);    # Hhe und Breite der Diagrammflche
   my ($xmin, $xmax);      # kleinster und grter x-Wert
   my ($ymin, $ymax);      # kleinster und grter y-Wert

   # Kennwerte einlesen
   print "Beginne mit Einlesen der Daten aus Datei $datei.\n";
   open(DATEN, $datei) || die("Konnte Datei $datei nicht ffnen : $!\n");
   <DATEN>; # Tabellenkopf

   while (<DATEN>) { # Lese jetzt die Werte ein
      chomp;
      @data=split(/;/); #/
      push(@xwert, $data[1]);
      push(@ywert, $data[2]);}
   close(DATEN);

   # Jeweils Minimal- und Maximalwert fr x- und y-Werte bestimmen
   $xmin=-10;
   $xmax=10;
   $ymin=-10;
   $ymax=10;
   for ($e=0; $e<@xwert; $e++) {
      $xmin=$xwert[$e] if ($xmin>$xwert[$e]);
      $xmax=$xwert[$e] if ($xmax<$xwert[$e]);
      $ymin=$ywert[$e] if ($ymin>$ywert[$e]);
      $ymax=$ywert[$e] if ($ymax<$ywert[$e]);}

   # Jetzt einige Koordinaten und Faktoren berechnen
   $nullx=$randl;
   $nully=$hoehe-$randu;
   $tiefe=$hoehe-$rando-$randu;
   $weite=$breite-$randl-$randr;
   $xscale=$weite/($xmax-$xmin);
   $yscale=$tiefe/($ymax-$ymin);

   # Jetzt das Bild vorbereiten
   print "Zeichne Diagramm Terminabweichung.\n";
   $bild=GD::Image->new($breite, $hoehe);
   $color{"weiss"}  = $bild->colorAllocate(255,255,255); # wei ist automatisch Hintergrundfarbe
   $color{"schwarz"}= $bild->colorAllocate(0,0,0);
   $color{"blau"}   = $bild->colorAllocate(0,0,200);
   $color{"rot"}    = $bild->colorAllocate(200,0,0);
   $color{"gruen"}  = $bild->colorAllocate(0,200,0);
   $color{"gelb"}   = $bild->colorAllocate(0,150,150);

   # Rahmen zeichnen
   $bild->rectangle($nullx, $rando, $nullx+$weite, $nully, $color{"schwarz"});
   $bild->rectangle($nullx+1, $rando+1, $nullx+$weite-1, $nully-1, $color{"schwarz"});

   # Achsen zeichnen
   $a=$nully-int($yscale*(0-$ymin));
   $bild->line($nullx, $a, $nullx+$weite, $a, $color{"schwarz"});  # x-Achse
   $a=$nullx+int($xscale*(0-$xmin));
   $bild->line($a, $rando, $a, $nully, $color{"schwarz"}); # y-Achse

   # Skalen zeichnen
   $d=$nully-int($yscale*(0-$ymin)); # die Hhe der x-Achse (Position der x-Achse auf der y-Achse)
   $e=$nullx+int($xscale*(0-$xmin)); # die Weite der y-Achse (Position der y-Achse auf der x-Achse)
   for ($a=0; $a<11; $a++) { # Skalen zeichnen, je 10 Teilstriche
      $b=$nully-int($tiefe*($a/10)); #/ # Position fr den nchsten Skalenstrich y-Achse
      $c=$nullx+int($weite*($a/10)); #/ # Position fr den nchsten Skalenstrich x-Achse
      $bild->line($c, $d, $c, $d+10, $color{"schwarz"});  # x-Achse
      $bild->line($e-10, $b, $e, $b, $color{"schwarz"});} # y-Achse

   # Achsenbeschriftung
   $bild->stringUp(gdSmallFont, int($randl/2)-12, int($hoehe/2)-45 ,"Verzgerung [d]", $color{"blau"}); # y(1)-Achse, links
   $bild->string(gdSmallFont, int($breite/2)-96, $hoehe-int($randu/2), "rel. Zugangsterminabweichung [d]", $color{"schwarz"}); # x-Achse, unten

   # Diagrammbeschriftung
   $bild->string(gdLargeFont, int($breite/2)-64, int($rando/2)-8, "Termineinhaltung", $color{"schwarz"});

   # Skalenbeschriftung
   # Beachte : $d und $e brauchen gleiche Werte wie beim Zeichnen der Skalen
   for ($a=0; $a<11; $a++) {
      $b=$nully-int($tiefe*($a/10)); #/ # Position des nchsten Teilstriches y-Achse
      $c=$nullx+int($weite*($a/10)); #/ # Position des nchsten Teilstriches x-Achse
      $f=int(($xmax-$xmin)*($a/10)+$xmin); #/
      $bild->string(gdSmallFont, $c-10, $d+12, "$f", $color{"schwarz"}); # x-Skala
      $f=int(($ymax-$ymin)*($a/10)+$ymin); #/
      $bild->stringUp(gdSmallFont, $e-27, $b+10, "$f", $color{"schwarz"});} # y-Skala

   # Grenbestimmung fr die Punkte
   $d=$linie1; # Ausgangswert ist Linienbreite 1
   $d=int($xscale-2) if ($d>$xscale); # wenn $d breiter als eine x-Einheit, dann verkleinern
   $d=int($yscale-2) if ($d>$yscale); # wenn $d   "      "    "  y-   "   ,   "       "
   $d=1 if ($d<1); # $d auf 1 Pixel setzen, wenn es kleiner als ein Pixel geworden ist

   # Nun die Punkte zeichnen
   for ($a=0; $a<@xwert; $a++) {
      $b=$nullx+int(($xwert[$a]-$xmin)*$xscale); # Werte in Pixel umrechnen
      $c=$nully-int(($ywert[$a]-$ymin)*$yscale);
      $bild->arc($b, $c, $d, $d, 0, 360, $color{"blau"});
      $bild->fill($b, $c, $color{"blau"}); # den Kreis ausfllen
      }

   # Ausgabe des Diagramms
   $a=$dir."terminabweichung.png";
   print "Speichere Diagramm Terminabweichung in Datei $a\n";
   open(OUTPUT, ">$a") || die("Konnte Datei $a nicht ffnen : $!\n");
   binmode OUTPUT;
   print OUTPUT $bild->png;
   close(OUTPUT);
   }

# ---------- Hauptprogramm ----------
print <<ANNOUNCE;
DrawTA v1.0.2 - zeichnet Diagramm zur Terminabweichung
Copyright (C) 2001, 2002 Dirk Swiniartzki pamela\@planet-dirk.de

Dieses Programm ist freie Software. Sie knnen es unter
den Bedingungen der GNU General Public License Version 2,
wie von der Free Software Foundation herausgegeben, weitergeben
und/oder modifizieren.

Die Verffentlichung dieses Programms erfolgt in der
Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
ANNOUNCE

print "\nSyntax : drawta.plx <Verzeichnis>\n";

@config=init(); # @config= siehe init();
if (@ARGV>0) {
   $b=$ARGV[0];}
else {
   $b="./";}
$b.="terminabweichung.csv";
drawta(@config, $b);

__END__


