#!/usr/bin/perl -w

use strict;

# PameLa v1.0.2 - Programm fr die automatisierte engpaorientierte Logistikanalyse
# Copyright (C) 2001, 2002 Dirk Swiniartzki (pamela@planet-dirk.de)
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License version 2
#    as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#    General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#    -----------------------------------------------------------
#    Dieses Programm ist freie Software. Sie knnen es unter
#    den Bedingungen der GNU General Public License Version 2,
#    wie von der Free Software Foundation herausgegeben, weitergeben
#    und/oder modifizieren.
#
#    Die Verffentlichung dieses Programms erfolgt in der
#    Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
#    GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
#    der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
#    Details finden Sie in der GNU General Public License.
#
#    Sie sollten eine Kopie der GNU General Public License zusammen
#    mit diesem Programm erhalten haben. Falls nicht, schreiben Sie
#    an die Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
#    MA 02139, USA.
#    ----------------------------------------------------------------

my @config;      # Liste der Konfigurationsparameter
my @systems;     # Liste der Arbeitssysteme
my ($min, $max); # erster & letzter vorkommender Tag
my %calendar;    # nimmt den Betriebskalender und die Kapazitten auf

#---------- init ----------
# initialisiert Programm
# Parameter : keine
# Rckgabewerte : Auftragsdatei, Arbeitsplandatei, Ergenbnisverzeichnis, Startdatum, Enddatum, Referenzdatum, Klasseneinteilung, Kalenderdatei, Pnktlichkeitsma

sub init {
   my $orderfile;    # Auftragsdatei
   my $workplanfile; # Arbeitsplandatei
   my $indexfile;    # Kennzahldatei
   my $resultdir;    # Verzeichnisname fr Ergebnisse
   my $startdate;    # Beginn Untersuchungszeitraum
   my $enddate;      # Ende Untersuchungszeitraum
   my $refdate;      # Referenzdatum
   my $classes;      # Klasseneinteilung Histogramme
   my $intime;       # Zeitfenster fr "pnklich"
   my $calendar;     # Kalenderdatei

   print "Beginne Konfiguration.\n";
   print "Suche Konfigurationsdatei...\n";

   if (@ARGV==1) {
      if (-e $ARGV[0]) {
         print "Konfigurationsdatei gefunden : $ARGV[0]\n";
         open(INPUT,"$ARGV[0]") || die("Konnte Konfigurationsdatei $ARGV[0] nicht ffnen : $!\n");
         while (<INPUT>) {
            chomp;
            next if (/#/);          # lines containing `#` are comments
            while (/ +/) { s/ +//;} # delete all spaces from configuration file
	    next if ($_ eq "");     # skip line if empty
            if (/^orderfile=/) {
               s/^orderfile=//;
               $orderfile=$_;
               next;}
            if (/^workplanfile=/) {
               s/^workplanfile=//;
               $workplanfile=$_;
               next;}
            if (/^resultdirectory=/) {
               s/^resultdirectory=//;
               $resultdir=$_;
               next;}
            if (/^startdate=/) {
               s/startdate=//;
               $startdate=$_;
               next;}
            if (/^enddate=/) {
               s/^enddate=//;
               $enddate=$_;
               next;}
            if (/^referencedate=/) {
               s/^referencedate=//;
               $refdate=$_;
               next;}
            if (/^classes=/) {
               s/^classes=//;
               $classes=$_;}
            if (/^intime=/) {
               s/intime=//;
               $intime=$_;}
            if (/^calendar=/) {
               s/^calendar=//;
               $calendar=$_;}}
         close(INPUT);}
      until (-e $orderfile) {
         print "Name der Auftragsdatei ? ";
         <STDIN>;
         chomp;
         $orderfile=$_;}
      until (-e $workplanfile) {
         print "Name der Arbeitsplandatei ? ";
         <STDIN>;
         chomp;
         $workplanfile=$_;}
      until (-e $resultdir) {
         print "In welchem Verzeichnis sollen die Ergebnisse gespeichert werden ? ";
         <STDIN>;
         chomp;
         $resultdir=$_;}
      until ($startdate) {
         print "Beginn Untersuchungszeitraum ? ";
         <STDIN>;
         chomp;
         $startdate=$_;}
      until ($enddate) {
         print "Ende Untersuchungszeitraum ? ";
         <STDIN>;
         chomp;
         $enddate=$_;}
      until ($refdate) {
         print "Referenzdatum ? ";
         <STDIN>;
         chomp;
         $refdate=$_;}
      until ($classes) {
         print "Klasseneinteilung in der Form \"10;20;30;40...\" eingeben.\n";
         print "Die Werte stellen dabei die rechte Klassengrenze (\"bis exklusive ...\") dar.\n";
         print "Klasseneinteilung ? ";
         <STDIN>;
         chomp;
         $classes=$_;}
      until ($intime) {
         print "Zeitfenster in der Form \"-3;1\" eingeben.\n";
         print "Zeitfenster fr pnktliche Auftrge ? ";
         <STDIN>;
         chomp;
         $intime=$_;}
      until (-e $calendar) {
         print "Name der Kalenderdatei ? ";
         <STDIN>;
         chomp;
         $calendar=$_;}}
   else {
      die("Falsche Syntax beim Aufruf von PameLa !\n");}
   print "Konfiguration abgeschlossen.\n\n";
   return($orderfile, $workplanfile, $resultdir, $startdate, $enddate, $refdate, $classes, $calendar, $intime);
}

#---------- convertdate ----------
# konvertiert Datum von tt.mm.jjjj in Dezimalzahl
# Parameter : Datum der Form tt.mm.jjjj
# returns : Dezimalzahl

sub convertdate {
   my ($date)=@_;
   my @days=(306, 337, 0, 31, 61, 92, 122, 153, 184, 214, 245, 275);
   my $day;
   my $month;
   my $year;
   my $a;

   $date=~/(\d{1,2})\.(\d{1,2})\.(\d{4})/; # Regex fr Datum der Form t[t].m[m].jjjj
   $day=$1;
   $month=$2;
   $year=$3;

   $year-- if ($month<3);
   $a=$day+$days[--$month]+$year*365+int($year/4)-int($year/100)+int($year/400);
   return $a;
}

#---------- get_wpdata ----------
# Liefert den fr einen Auftrag gltigen Arbeitsplaneintrag
# Parameter : Zugangsdatum, Referenz auf 2D-Array des Arbeitsplans
# Rckgabewerte : Basismenge, Rstzeit, Einzelzeit

sub get_wpdata {
   my ($date, $pointer)=@_;
   my ($a, $b);          # temporre Variablen
   my ($base, $tr, $te, $error); # Basismenge, Rst- und Einzelzeit, Fehler

   $b=@$pointer;
   for ($a=0; $a<$b;$a++) {
      if ($$pointer[$a][3]<=$date && $$pointer[$a][4]>=$date) {
         ($base, $tr, $te)=($$pointer[$a][0], $$pointer[$a][1], $$pointer[$a][2]); #@{[@$pointer[$a]]}[0..2];
	 last;}}
   return($base,$tr,$te);
}

#---------- get_ZDL_as_sec ----------
# Liefert die Durchlaufzeit in Sekunden
# Parameter : Startdatum(=Referenzdatum), Referenz auf Auftragsdatensatz, Arbeitsplatz
# Rckgabewert : die Durchlaufzeit in Sekunden

sub get_ZDL_as_sec {
   my ($startdate, $order, $workplace)=@_;
   my $a;     # temporre Variable
   my $secs;  # Die gesuchte Anzahl Sekunden

   if ($$order[5]==$$order[7]) { # Sonderfall : Zugang und Rckmeldung am gleichen Tag
      $secs=$$order[8]-$$order[6];}
   else {
      $secs=$calendar{$workplace}[($$order[5]*2)+1]-$$order[6];          # die verbleibende Zeit am Zugangstag
      $secs=$secs+($$order[8]-$calendar{$workplace}[$$order[7]*2]);      # die mitzurechnende Zeit am Rckmeldetag
      for ($a=(($$order[5]+1)*2); $a<($$order[7]*2); $a=$a+2) { # die Tage zwischen Freigabe und Rckmeldung
         $secs=$secs+ ($calendar{$workplace}[$a+1]) - ($calendar{$workplace}[$a]);}}
   return($secs);
}

#---------- get_ZDL_as_days ----------
# Liefert die Durchlaufzeit in Tagen
# Parameter : Zugangsdatum, Rckmeldedatum, Arbeitsplatz
# Rckgabewert : Die Durchlaufzeit in Tagen

sub get_ZDL_as_days {
   my ($start, $end, $workplace)=@_;
   my $a;    # temporre Variable
   my $days; # Die gesuchte Anzahl Tage

   $days=$end-$start; # Zuerst Ende- Start rechnen ...
   for ($a=($start+1)*2; $a<($end*2); $a=$a+2) { # ...dann fr jeden arbeitsfreien Tag eins abziehen
      $days-- if ($calendar{$workplace}[$a]==$calendar{$workplace}[$a+1]);}
   return($days);
}

#---------- calculate_ZAU ----------
# berechnet ZAU, ZDL und  (alles, wofr Arbeitsplan bentigt wird)
# Parameter : @config=($orderfile, $workplanfile, $resultdir, $startdate, $enddate, $refdate, $classes, $calendar, $intime)
# Rckgabewerte : Name der neuen Auftragsdatei, erster, letzter vorkommender Tag

sub calculate_ZAU {
   my @config=@_;
   my %plan;             # Nimmt den Arbeitsplan auf
   my @data;             # nimmt einen Datensatz (Arbeitsplan, spter Auftrge) auf
   my %system;           # Hash fr Arbeitssysteme
   my ($base, $tr, $te); # Basismenge, Rstzeit, Einzelzeit eines Auftrages
   my ($a, $b);          # temporre Variable
   my ($min, $max);      # Datum des ersten und des letzten vorkommenden Tages
   
   print "Lese Arbeitsplan ein... ";
   open(INPUT, $config[1]) || die("Konnte Arbeitsplandatei $config[1] nicht ffnen : $!\n");
   <INPUT>; # erste Zeile enthlt Tabellenkopf - bergehen
   while (<INPUT>) {
      chomp;
      @data=split(/;/, $_);  #/
      $data[6]=convertdate($data[6])-$config[5];
      $data[7]=convertdate($data[7])-$config[5];
      push(@{$plan{$data[0]}{$data[1]}{$data[2]}}, [@data[3..7]]);}
   close(INPUT);
   print "Arbeitsplan eingelesen.\n";

   print "Lese Arbeitszeitdatei ein... ";
   open(INPUT, $config[7]) || die("Konnte Arbeitszeitdatei $config[7] nicht ffnen : $!\n");
   <INPUT>; # erste Zeile=Tabellenkopf
   while (<INPUT>) {
      chomp;
      @data=split(/;/, $_); #/
      $a=shift(@data);
      foreach $b (@data) { # Konvertiere Zeitangabe [h]h:mm in Sekundenzahl
         $b=~/(\d{1,2}):(\d{2})/;
         $b=$1*3600+$2*60;}
      $calendar{$a}=[@data];}
   close(INPUT);
   print "Betriebskalender eingelesen.\n";

   print "Berechne Auftragszeit und konvertiere Datumsangaben.\n";
   open(INPUT, $config[0]) || die("Konnte Auftragsdatei $config[0] nicht ffnen : $!\n");
   open(OUTPUT, ">$config[0].neu") || die("Konnte Datei $config[0].neu nicht anlegen : $!\n");
   <INPUT>; # erste Zeile bergehen, da Tabellenkopf
   print OUTPUT "Auftragsnummer;Arbeitsvorgang;Arbeitsplatz;Material;Losgre;Zugangsdatum;Zugangszeit;Rckmeldedatum;Rckmeldezeit;Soll-Zugang;Soll-Rckmeldung;Auftragszeit;Durchlaufzeit(Tage);Durchlaufzeit(h)";

   $min=1000000; # Dummy-Wert, damit Vergleich in Schleife funktioniert
   $max=0;
   while(<INPUT>) {
      chomp;
      @data=split(/;/, $_); #/
      $data[5]=convertdate($data[5])-$config[5];   # Datumsangaben konvertieren
      $data[7]=convertdate($data[7])-$config[5];
      $data[9]=convertdate($data[9])-$config[5];
      $data[10]=convertdate($data[10])-$config[5];
      $min=$data[5] if ($min>$data[5]);            # ersten und letzten vorkommenden Tag bestimmen
      $max=$data[7] if ($max<$data[7]);
      foreach $a (6,8) { # konvertiere Zeitangabe in Sekundenzahl
         $data[$a]=~/(\d{1,2}):(\d{2}):(\d{2})/;
         $data[$a]=$1*3600+$2*60+$3;}
      $system{$data[2]}=1; # Arbeitssystem in Liste eintragen
      ($base, $tr, $te)=get_wpdata($data[5],$plan{$data[3]}{$data[1]}{$data[2]});
      $a=$data[4]/$base; # / # Zyklenanzahl bestimmen
      $a=int($a)+1 unless ($a==int($a)); # Wenn $a keine ganze Zahl, dann aufrunden
      $a=($tr+$a*$te)/60;  #/ #ZAU in h berechnen
      push(@data,$a);
      $a=get_ZDL_as_days($data[5], $data[7], $data[2]);  # ZDL in Tagen berechnen
      push(@data,$a);
      $a=get_ZDL_as_sec($config[5], \@data, $data[2]);
      $a/=3600; #/ ZDL in Stunden umrechnen
      push(@data, $a);
      $a=join(";", @data);#/
      print OUTPUT "$a\n";}

   close(INPUT);
   close(OUTPUT);
   print "Auftragszeitberechnung beendet.\n";
   $config[0].=".neu";
   @systems=sort(keys(%system));   # @systems ist globale Variable
   return($config[0], $min, $max);
}

#---------- calculate_DUDI ----------
# berechnet Daten fr Durchlaufdiagramme
# Parameter : erster, letzter vorkommender Tag, @config=($orderfile, $workplanfile, $resultdir, $startdate, $enddate, $refdate, $classes, $calendar, $intime)
# Rckgabewerte :

sub calculate_DUDI {
   my ($min, $max, @config)=@_;
   my %zugang;       # Zugang
   my %abgang;       # Abgang
   my ($a, $b, $c, $d);  # temporre Variablen
   my $kum_zugang;   # kumulierter Zugang
   my $kum_abgang;   # kumulierter Abgang
   my %kennzahlen;   # Liste der berechneten Kennzahlen : BImin, Bm, Lm, Lmax, ZAUm, ZAUs, Rm, ZDLmg(BKT), ZDLmg(h), Lges, Flugrad, Auslastung
   my @data;         # temporre Variable
   my %ordercount;   # Anzahl der Auftrge pro Arbeitssystem
   my %intimecount;  # Anzahl der pnklichen Auftrge
   my %histo_ZAU;    # Nimmt die ZAU-Verteilungen auf
   my %histo_ZDL;    # ZDL-Verteilungen
   my ($intime_first, $intime_last); # Pnklicher Auftrag -  Beginn, Ende
   my @classes;      # Nimmt die Klasseneinteilung auf
   my $bestand;      # fr die Berechnung des mittleren Bestandes

   # Vorbereitungen zur Datenauswertung
   print "Beginne mit Berechnung der Durchlaufdiagramme... ";
   ($intime_first, $intime_last)=split(/;/, $config[8]); #/
   @classes=split(/;/, $config[6]); #/
   $b=$max-$min+1;
   for ($a=0; $a<$b; $a++) { # Bereite ein leeres Array der Gre des Zeitraumes aller Daten vor
      $data[$a]=0;}
   foreach $a (@systems) {   # Flle Zu- und Abgang sowie Kennzahlen mit Nullen auf
      $zugang{$a}=[@data];
      $abgang{$a}=[@data];
      $kennzahlen{$a}=[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]; # 13 Kennzahlen, also 13 Nullen
      $intimecount{$a}=0;
      $ordercount{$a}=0;}

   @data=();
   for ($a=0; $a<=@classes; $a++) {       # leeres Array entsprechend der Anzahl Histogrammklassen anlegen
      $data[$a]=0;}
   foreach $a (@systems) {               # histo_ZAU und histo_ZDL mit Nullen fllen
      $histo_ZAU{$a}=[@data];
      $histo_ZDL{$a}=[@data];}

   # Jetzt Auftragsdaten verarbeiten
   open(INPUT, $config[0]) || die("Konnte Auftragsdatei $config[0] nicht ffnen : $!\n");
   <INPUT>;
   while (<INPUT>) {
      chomp;
      @data=split(/;/, $_);#/
      $zugang{$data[2]}[$data[5]-$min]+=$data[11];
      $abgang{$data[2]}[$data[7]-$min]+=$data[11];
      next if ($data[5]<$config[3] || $data[5]>$config[4]); # Wenn Zugang auerhalb des Untersuchungszeitraumes liegt, keine Bercksichtigung bei Kennzahlberechnung
      $ordercount{$data[2]}++;                       # Anzahl der Auftrge um 1 erhhen
      $a=$data[7]-$data[10];                         # Terminabweichung Abgang temp. errechnen
      $intimecount{$data[2]}++ if ($a>=$intime_first && $a<=$intime_last); # Wenn der Auftrag pnklich ist, zhlen
      $kennzahlen{$data[2]}[0]+=$data[11]**2;        # Bilden der Summe ZAU^2
      $kennzahlen{$data[2]}[4]+=$data[11];           #   "     "    "   ZAU
      $kennzahlen{$data[2]}[7]+=$data[11]*$data[12]; #   "     "    "   ZDL(BKT)*ZAU
      $kennzahlen{$data[2]}[8]+=$data[11]*$data[13]; #   "     "    "   ZDL(h)*ZAU
      $b=0;
      for ($a=0; $a<@classes; $a++) {                # Ermittle die Klasse fr ZAU
         if ($data[11]<$classes[$a]) {
            $histo_ZAU{$data[2]}[$a]++;
            $b=1;
            last;}}
      $histo_ZAU{$data[2]}[-1]++ unless($b);         # Wenn ZAU grer als letzte Klassengrenze, dann in Klasse "ber..." verbuchen
      $b=0;
      for ($a=0; $a<@classes; $a++) {                # Ermittle die Klasse fr ZDL (in Tagen)
         if ($data[12]<$classes[$a]) {
            $histo_ZDL{$data[2]}[$a]++;
            $b=1;
            last;}}
      $histo_ZDL{$data[2]}[-1]++ unless ($b);}

   close(INPUT);
   print "Durchlaufdiagramme berechnet.\n";

   # Als nchstes die Kennzahlberechnungen :
   print "Berechne Kennzahlen... ";
   foreach $a (@systems) {
      $kennzahlen{$a}[0]/=$kennzahlen{$a}[4]; #/ # Berechne BImin
      $kennzahlen{$a}[7]/=$kennzahlen{$a}[4]; #/ # Berechne ZDLmg(BKT)
      $kennzahlen{$a}[8]/=$kennzahlen{$a}[4]; #/ # Berechne ZDLmg(h)
      $kennzahlen{$a}[4]/=$ordercount{$a};    #/ # Berechne ZAUm
      $kennzahlen{$a}[5]=sqrt($kennzahlen{$a}[0]*$kennzahlen{$a}[4]-$kennzahlen{$a}[4]**2); # Berechne ZAUs
      $bestand=0;
      for ($b=0; $b<$config[3]-$min; $b++) { # den Bestand am Beginn des Untersuchungszeitraumes berechnen
         $bestand=$bestand+$zugang{$a}[$b]-$abgang{$a}[$b];}
      $c=$config[4]-$config[3]+1; # Berechne den Untersuchungszeitraum
      for ($b=($config[3]-$min); $b<=($config[4]-$min); $b++) {# Untersuchungszeitraum ist exklusive Start, inklusive Ende
         unless ($calendar{$a}[$b*2]==$calendar{$a}[$b*2+1]) { # Nur Tage, an denen gearbeitet wird, drfen bercksichtigt werden
            $kennzahlen{$a}[9]+=$abgang{$a}[$b];               # Berechne Lgesamt
            $bestand=$bestand+$zugang{$a}[$b]-$abgang{$a}[$b];
	    $kennzahlen{$a}[1]+=$bestand;                      # Berechne Summe B
	    $kennzahlen{$a}[3]=$abgang{$a}[$b] if ($kennzahlen{$a}[3]<$abgang{$a}[$b]); # Ermittle Lmax
            $kennzahlen{$a}[11]=$kennzahlen{$a}[11]+($calendar{$a}[$b*2+1]-$calendar{$a}[$b*2])/3600;} #/ # Berechne die Gesamtarbeitszeit
         else { $c--;}} # reduziere die Anzahl der Tage um die Tage, wo nicht gearbeitet wird
      $kennzahlen{$a}[1]/=$c;  #/ # Berechne Bm
      $kennzahlen{$a}[2]=$kennzahlen{$a}[9]/$c;  #/ # Berechne Lm
      $kennzahlen{$a}[11]=($kennzahlen{$a}[9]/$kennzahlen{$a}[11])*100; #/ # Berechne Auslastung
      $kennzahlen{$a}[6]=$kennzahlen{$a}[1]/$kennzahlen{$a}[2];         #/ # Berechne Rm
      $kennzahlen{$a}[10]=$kennzahlen{$a}[1]/$kennzahlen{$a}[0];        #/ # Berechne Flugrad FG
      $kennzahlen{$a}[12]=($intimecount{$a}/$ordercount{$a})*100;}      #/ # Berechne Anteil der pnklichen Auftrge

   # Nun die Ergebnisse ausgeben :
   open(KENNZAHL, ">$config[2]kennzahlen.csv") || die("Konnte Kennzahldatei $config[2]kennzahl.csv nicht anlegen : $!\n");
   print KENNZAHL "Arbeitssystem;BImin;Bm;Lm;Lmax;ZAUm;ZAUs;Rm;ZDLmg (in BKT);ZDLmg (in h);Lgesamt;Flugrad;Auslastung (%);Pnklich (%)\n";
   foreach $a (@systems) {

      # Kennzahl-Datensatz ausgeben
      $b=join(";", @{$kennzahlen{$a}});#/
      print KENNZAHL "$a;$b\n";

      # DUDI-Daten ausgeben
      open(OUTPUT, ">$config[2]dudi_$a.csv") || die("Konnte Datei $config[2]dudi_$a.csv nicht anlegen : $!\n");
      print OUTPUT "Durchlaufdiagramm fr Arbeitssystem : $a\n\n";
      print OUTPUT "Tag;Zugang;Abgang;Zugang kumuliert;Abgang kumuliert;Bestand\n";
      $kum_zugang=$kum_abgang=0;
      for ($c=0; $c<=($max-$min); $c++) {
         $kum_zugang+=$zugang{$a}[$c];
         $kum_abgang+=$abgang{$a}[$c];
         $b=($c+$min).";$zugang{$a}[$c];$abgang{$a}[$c];$kum_zugang;$kum_abgang;".($kum_zugang-$kum_abgang)."\n";
         print OUTPUT "--- Beginn Untersuchungszeitraum ---\n" if (($c+$min)==$config[3]);
         print OUTPUT $b;
         print OUTPUT "--- Ende Untersuchungszeitraum ---\n" if (($c+$min)==$config[4]);}
      close(OUTPUT);

      # Histogramm-Verteilungen ausgeben
      open(HISTO1, ">$config[2]histo_ZAU_$a.csv") || die("Konnte Datei $config[2]histo_ZAU_$a.csv nicht anlegen : $!\n");
      open(HISTO2, ">$config[2]histo_ZDL_$a.csv") || die("Konnte Datei $config[2]histo_ZDL_$a.csv nicht anlegen : $!\n");
      print HISTO1 "Histogramm der Auftragszeit ZAU fr Arbeitssystem $a\n\n";
      print HISTO1 "Klasse;Hufigkeit\n";
      print HISTO2 "Histogramm der Durchlaufzeit ZDL fr Arbeitssystem $a\n\n";
      print HISTO2 "Klasse; Hufigkeit\n";
      for ($c=0; $c<@classes; $c++) {
         print HISTO1 "bis ausschlielich $classes[$c];$histo_ZAU{$a}[$c]\n";
         print HISTO2 "bis ausschlielich $classes[$c];$histo_ZDL{$a}[$c]\n";}
      print HISTO1 "$classes[-1] und hher;$histo_ZAU{$a}[-1]\n";
      print HISTO2 "$classes[-1] und hher;$histo_ZDL{$a}[-1]\n";
      close(HISTO1);
      close(HISTO2);}

   print "Kennzahlen berechnet.\n";
   close(KENNZAHL);
}

#---------- calculate_PKL ----------
# berechnet Daten fr Produktionskennlinien
# Parameter : @config=($orderfile, $workplanfile, $resultdir, $startdate, $enddate, $refdate, $classes, $calendar, $intime)
# Rckgabewerte : keine

sub calculate_PKL {
   my @config=@_;
   my ($a, $b, $c, $d, $t);  # temporre Variable
   my @data;                 # nimmt Kennzahlen-Datensatz auf

   print "Beginne mit Berechnung der Produktionskennlinien.\n";
   open(INPUT, "$config[2]kennzahlen.csv") || die("Konnte Datei $config[2]kennzahlen.csv nicht ffnen : $!\n");
   <INPUT>; # erste Zeile enthlt Tabellenkopf, bergehen
   while (<INPUT>) {
      chomp;
      @data=split(/;/); #/
      open(OUTPUT, ">$config[2]pkl_$data[0].csv") || die("Konnte Datei pkl_$data[0].csv nicht ffnen : $!\n");
      print OUTPUT "Arbeitssystem : $data[0]\nLeistung : $data[3]\nidealer Mindestbestand : $data[1]\n\n";
      print OUTPUT "t;Bm(t);Lm(t);Rm(t)\n";
      print "Berechne Produktionskennlinie fr Arbeitssystem $data[0].\n";
#      foreach $t (0.0000001, 0.000001, 0.00001, 0.0001, 0.001) { # Den Anfang der PKL berechnen, damit's kein Loch gibt
#         $a=1-(1-$t**0.25)**4;              # Punkte der Kennlinie berechnen
#         $b=($data[1]*$a)+($data[1]*10*$t);
#         $c=$data[3]*$a;
#         if ($c) {
#            $a=$b/$c;}
#         else {
#            $a="---";}
#         print OUTPUT "$t;$b;$c;$a\n";}
      for ($d=1;$d<401;$d++) {
         $t=$d*0.0025;
         $a=1-(1-$t**0.25)**4;           # Punkte der Kennlinie berechnen
         $b=($data[1]*$a)+($data[1]*10*$t);
         $c=$data[3]*$a;
         if ($c) {
            $a=$b/$c;} #/
         else {
            $a="---";}
         print OUTPUT "$t;$b;$c;$a\n";}
      close(OUTPUT);}
   print "Berechnung der Produktionskennlinien abgeschlossen.\n";
   close(INPUT);
}

#---------- check_wpdata ----------
# Liefert den fr einen Auftrag gltigen Arbeitsplaneintrag, ansonsten Fehler
# Parameter : Datum, Referenz auf 2D-Array
# Rckgabewerte : Basismenge, Rstzeit, Einzelzeit, Fehlerstatus

sub check_wpdata {
   my ($date, $pointer)=@_;
   my ($a, $b);          # temporre Variablen
   my ($base, $tr, $te, $error); # Basismenge, Rst- und Einzelzeit, Fehlerstatus

   $error=1;
   $b=@$pointer;
   for ($a=0; $a<$b;$a++) {
      if ($$pointer[$a][3]<=$date && $$pointer[$a][4]>=$date) {
         ($base, $tr, $te)=($$pointer[$a][0], $$pointer[$a][1], $$pointer[$a][2]);
         $error=0;           # Fehlerindikator lschen, wenn gefunden
         $$pointer[$a][5]=1; # Datensatz als "benutzt" markieren
         last;}}
   return($base,$tr,$te, $error);
}

#---------- checkdata ----------
# berprft Daten auf Konsistenz
# Parameter : @config
# Rckgabewerte : @config

sub checkdata {
   my @config=@_;
   my ($a, $b, $c, $d, $e); # temporre Variablen
   my @data;                # Array fr temporre Datenstze
   my @temp;                # temporres Array
   my $zeile;               # Zeilenangabe, um Datenfehler besser zu finden
   my %plan;                # nimmt den Arbeitsplan auf

   %calendar=(); # %calendar ist globale Variable, lschen, damit es keine Probleme gibt
   print "Beginne mit Prfung der Daten.\n";
   print "Die Prfergebnisse werden in der Datei $config[2]log.txt gespeichert.\n";
   open(LOG, ">$config[2]log.txt") || die("Konnte Datei $config[2]log.txt nicht anlegen : $!\n");
   print LOG "Ergebnisse der Datenprfung - benutzte Konfiguration :\n";
   print LOG "Arbeitsplandatei : $config[1]\n";
   print LOG "Auftragsdatei : $config[0]\n";
   print LOG "Kalenderdatei : $config[7]\n\n";

   print "Beginne mit Prfung des Arbeitsplans...\n";
   print LOG "Beginne mit Prfung des Arbeitsplans...\n";

   # Arbeitsplan einlesen und prfen
   open(INPUT, $config[1]) || die("Konnte Arbeitsplandatei $config[1] nicht ffnen : $!\n");
   <INPUT>; # erste Zeile enthlt Tabellenkopf - bergehen
   @temp=qw(Material Arbeitsvorgang Arbeitsplatz Basismenge Rstzeit Einzelzeit gltig_ab gltig_bis);

   $zeile=1;
   while (<INPUT>) {
      chomp;
      $zeile++;
      @data=split(/;/, $_); #/

      # einzelnen Datensatz aus Arbeitsplan prfen
      if (@data<7) { # Prfe, ob Datensatz vollstndig
         print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => unvollstndiger Datensatz.\n";
         print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => unvollstndiger Datensatz.\n";
         while (@data<7) {push(@data, 0);}} # Dummywerte einfgen
      for ($a=0; $a<8; $a++) { # Prfe, ob jeder nicht leer ist
         if ($data[$a] eq "") {
            print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] fehlt.\n";
            print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] fehlt.\n";
            $data[$a]=0 if ($a<6);  # Dummy-Werte einsetzen, um Prfung fortsetzen zu knnen :
            $data[$a]="1.1.2000" if ($a>5);}} # 0 fr Zahlenwerte, 1.1.2000 fr Datumswerte

      foreach $a (4,5) { # Prfe, ob Eintrge fr Rstzeit und Einzelzeit eine Zahl sind
         unless ($data[$a]=~/\d*/ || $data[$a]=~/\d*\.\d*/) {
            print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => $temp[$a] ist keine Zahl.\n";
            print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => $temp[$a] ist keine Zahl.\n";
            $data[$a]=1;}}

      foreach $a (6,7) { # Prfe, ob Eintrge fr gltig_ab und gltig_bis Datumsformat haben
         unless ($data[$a]=~/\d{1,2}\.\d{1,2}\.\d{4}/) {
            print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig bis $data[7] => $temp[$a] ist kein Datum.\n";
            print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig bis $data[7] => $temp[$a] ist kein Datum.\n";
            $data[$a]="1.1.2000";}}

      unless ($data[3]=~/\d*/) { # Basismenge mu ganze Zahl sein
         print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Basismenge ist keine ganze Zahl.\n";
         print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Basismenge ist keine ganze Zahl.\n";
         $data[3]=int($data[3]);}
      if ($data[3]<1) { # Basismenge mu mindestens eins sein
         print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Basismenge kleiner 1.\n";
         print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Basismenge kleiner 1.\n";
         $data[3]=1;}
      if ($data[4]<0) { # Rstzeit darf nicht kleiner Null sein
         print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Rstzeit ist kleiner Null.\n";
         print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Rstzeit ist kleiner Null.\n";
         $data[4]=0;}
      if ($data[5]<=0) { # Einzelzeit mu grer Null sein
         print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Einzelzeit ist kleiner oder gleich Null.\n";
         print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => Einzelzeit ist kleiner oder gleich Null.\n";
         $data[5]=1;}

      # Datum konvertieren
      $data[6]=convertdate($data[6])-$config[5];
      $data[7]=convertdate($data[7])-$config[5];
      if ($data[6]>$data[7]) {
         print "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => gltig_bis liegt nach gltig_ab.\n";
         print LOG "Zeile $zeile : Material $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] - gltig ab $data[6] - gltig bis $data[7] => gltig_bis liegt nach gltig_ab.\n";}
      $data[8]=0; # fr Kennzeichen, ob Datensatz benutzt wurde
      push(@{$plan{$data[0]}{$data[1]}{$data[2]}}, [@data[3..8]]);}
   close(INPUT);

   # Jetzt Inkonsistenzen zwischen unterschiedlichen Datenstzen des Aplans testen
   # Prfen, ob berschneidungen zwischen einzelnen Datenstzen eines Aplans vorhanden
   foreach $a (keys %plan) {
      foreach $b (keys %{$plan{$a}} ) {
         foreach $c (keys %{$plan{$a}{$b}} ) {
            @data=@{$plan{$a}{$b}{$c}};  # @data nimmt die Referenzen auf die einzelnen Aplan-Datenstze auf
            for ($d=0; $d<@data; $d++) {
               for ($e=$d+1; $e<@data; $e++) {
                  unless ($data[$d][4]<$data[$e][3] || $data[$d][3]<$data[$e][4]) { # Start1>Stop2 oder Start2>Stop1 ist die Bedingung, damit es keine berschneidungen gibt
                     print "Material $a - Arbeitsvorgang $b - Arbeitsplatz $c => zeitliche berschneidungen gefunden.\n";
                     print LOG "Material $a - Arbeitsvorgang $b - Arbeitsplatz $c => zeitliche berschneidungen gefunden.\n";}}}}}}

   print "Prfung des Arbeitsplans abgeschlossen.\n\n";
   print LOG "Prfung des Arbeitsplans abgeschlossen.\n\n";

   # Betriebskalender wird nicht geprft, das soll das Kalenderprogramm machen (oder doch hier?)
   # Betriebskalender einlesen
   print "Lese Betriebskalender ein ...\n";
   open(INPUT, $config[7]) || die("Konnte Kalenderdatei $config[7] nicht ffnen : $!\n");
   <INPUT>; # erste Zeile=Tabellenkopf
   while (<INPUT>) {
      chomp;
      @data=split(/;/, $_); # /
      $a=shift(@data);
      foreach $b (@data) { # Konvertiere Zeitangabe [h]h:mm in Sekundenzahl
         $b=~/(\d{1,2}):(\d{2})/;
         $b=$1*3600+$2*60;}
      $calendar{$a}=[@data];}  #push(@{$calendar{$a}}, [@data]);}
   close(INPUT);
   print "Betriebskalender eingelesen.\n\n";

   # Jetzt die Rckmeldungen prfen
   print "Beginne mit Prfung der Auftragsdaten...\n";
   print LOG "Beginne mit Prfung der Auftragsdaten :\n";
   open(MELDUNG, $config[0]) || die("Konnte Datei $config[0] nicht ffnen : $!\n.");
   <MELDUNG>; # erste Zeile enthlt Tabellenkopf

   @temp=qw(Auftragsnummer Arbeitsvorgang Arbeitsplatz Material Losgre Zugangsdatum Zugangszeit Rckmeldedatum Rckmeldezeit Sollzugang Sollrckmeldung);
   $zeile=0;
   while (<MELDUNG>) {
      chomp;
      $zeile++;
      @data=split(/;/, $_); #/

      # einzelnen Datensatz prfen
      if (@data<11) {  # Datensatz vollstndig ?
         print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => unvollstndiger Datensatz.\n";
         print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => unvollstndiger Datensatz.\n";
         while (@data<11) {push(@data, 0);}} # Dummywerte einfgen
      for ($a=0; $a<11; $a++) {
         if ($data[$a] eq "") { # Alle Elemente mit Werten gefllt ?
            print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] fehlt.\n";
            print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] fehlt.\n";
            $data[$a]=0;}}
      unless ($data[4]=~/\d*/) { # Ist Losgre eine ganze Zahl
         print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => Losgre ist keine ganze Zahl.\n";
         print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => Losgre ist keine ganze Zahl.\n";}
      foreach $a (5,7,9,10) { # Sind Freigabe, Rckmeldung, Sollfreigabe und Sollrckmeldung Datumsangaben ?
         unless ($data[$a]=~/(\d{1,2})\.(\d{1,2})\.(\d{4})/) {
            print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] ist kein Datum.\n";
            print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] ist kein Datum.\n";
            $data[$a]="1.1.2000";}}
      foreach $a (6,8) { # Sind Freigabezeit und Rckmeldezeit im richtigen Format ?
         unless ($data[$a]=~/\d{1,2}:\d{2}:\d{2}/) {
            print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] ist keine Zeitangabe.\n";
            print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => $temp[$a] ist keine Zeitangabe.\n";
            $data[$a]="12:00";}}
      # Grundlegende Tests abgeschlossen, jetzt Datums- und Zeitkonvertierung
      $data[5]=convertdate($data[5])-$config[5];
      $data[7]=convertdate($data[7])-$config[5];
      $data[9]=convertdate($data[9])-$config[5];
      $data[10]=convertdate($data[10])-$config[5];
      foreach $a (6,8) { # konvertiere Zeitangabe in Sekundenzahl
         $data[$a]=~/(\d{1,2}):(\d{2}):(\d{2})/;
         $data[$a]=$1*3600+$2*60+$3;}
      # Test, ob Rckmeldung-Freigabe >= ZAU und ob passender Arbeitsplaneintrag vorhanden
      ($a, $b, $c, $d)=check_wpdata($data[5],$plan{$data[3]}{$data[1]}{$data[2]});
      if ($d) {  # Fehler aufgetreten
         print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => kein gltiger Arbeitsplan-Datensatz vorhanden.\n";
         print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => kein gltiger Arbeitsplan-Datensatz vorhanden.\n";}
      else { # Datensatz im Arbeitsplan vorhanden, also Vergleich ZAU - ZDL
         $e=$data[4]/$a; #/# Anzahl Zyklen berechnen
         $e=int($e)+1 if ($e!=int($e));
         $d=($b+$e*$c)/60; #/# ZAU in h berechnen
         $a=get_ZDL_as_sec($config[5], \@data, $data[2]); # ZDL in sek berechnen
         $a/=3600; #/
         if ($d>$a) {
            #print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => Auftragszeit $d > Durchlaufzeit $a.\n";
            print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => Auftragszeit $d > Durchlaufzeit $a.\n";}
         # Die Soll-ZDL in h berechnen :
         $a=0;
	 for ($b=$data[9]; $b<($data[10]+1); $b++) { # die Tage zwischen Zugang und Rckmeldung hinzuzhlen
	    $a=$a+$calendar{$data[2]}[$b*2+1]-$calendar{$data[2]}[$b*2];}

	 $a/=3600; #/      # ZDL auf Stunden umrechnen, intern wird in Sekunden gerechnet
         if ($a<$d) {
            #print "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => Auftragszeit > Soll-Durchlaufzeit.\n";
            print LOG "Zeile $zeile : Auftragsnummer $data[0] - Arbeitsvorgang $data[1] - Arbeitsplatz $data[2] => Auftragszeit $d > Soll-Durchlaufzeit $a.\n";}}}

   close(MELDUNG);
   print "Auftragsdaten berprft.\n\n";
   print LOG "Auftragsdaten berprft.\n\n";

   # berflssige Arbeitsplandaten ausgeben
   print "Folgende Datenstze des Arbeitsplanes wurden nicht benutzt :\n";
   print LOG "Folgende Datenstze des Arbeitsplanes wurden nicht benutzt :\n";
   open(APLAN, ">$config[1].neu") || die("Konnte Datei $config[1].neu nicht ffnen : $!\n");
   print APLAN "Material;Arbeitsvorgang;Arbeitsplatz;Basismenge;Rstzeit;Einzelzeit;gltig ab;gltig bis\n";

   foreach $a (keys %plan) {
      foreach $b (keys %{$plan{$a}} ) {
         foreach $c (keys %{$plan{$a}{$b}} ) {
            @data=@{$plan{$a}{$b}{$c}};
            for ($d=0; $d<@data; $d++) {
               unless ($data[$d][5]) {
                  print "Material $a - Arbeitsvorgang $b - Arbeitsplatz $c - gltig ab $data[$d][3] - gltig bis $data[$d][4]\n";
                  print LOG "Material $a - Arbeitsvorgang $b - Arbeitsplatz $c - gltig ab $data[$d][3] - gltig bis $data[$d][4]\n";}
               else {
                  print APLAN "$a;$b;$c;$data[$d][0];$data[$d][1];$data[$d][2];$data[$d][3];$data[$d][4]\n";}}}}}
   close(APLAN);
   print "\nDie ungenutzten Datenstze wurden aus dem Arbeitsplan gelscht.\n";
   print "Die neue Arbeitsplandatei heit $config[1].neu .\n\n";
   print LOG "\nDie ungenutzten Datenstze wurden aus dem Arbeitsplan gelscht.\n";
   print LOG "Die neue Arbeitsplandatei heit $config[1].neu .\n\n";
   print "Datenprfung abgeschlossen.\nDas Fehlerprotokoll finden Sie in der Datei $config[2]log.txt .\n";
   print LOG "Datenprfung abgeschlossen.\n";
   close(LOG);
   return(@config);
}

#---------- calculate_TA ----------
# Berechnet die Terminabweichung SOLL-IST
# Parameter : @config
# Rckgabewerte :

sub calculate_TA {
   my @config=@_;
   my ($a, $b, $c);  # temporre Variablen
   my @data;         # nimmt den aktuellen Datensatz auf

   print "Beginne mit der Berechnung der Terminabweichungen...\n";
   open(INPUT, $config[0]) || die("Konnte Datei $config[0] nicht ffnen : $!\n");
   open(OUTPUT, ">$config[2]terminabweichung.csv") || die("Konnte Datei $config[2]terminabweichung.csv nicht ffnen : $!\n");
   <INPUT>; # erste Zeile enthlt Tabellenkopf
   print OUTPUT "Auftragsnr.;relative Zugangsterminabweichung;Verzgerung\n";

   while(<INPUT>) {
      chomp;
      @data=split(/;/, $_); #/
      $a=$data[5]-$data[9]; # relative Zugangsterminabweichung
      $b=($data[7]-$data[5])-($data[10]-$data[9]); # relative Verzgerung
      print OUTPUT "$data[0];$a;$b\n";}

   close(INPUT);
   close(OUTPUT);
   print "Terminabweichungen berechnet.\n";
}

#---------- Hauptprogramm -----------
print <<ANNOUNCE;
PameLa v1.0.2 - Programm fr die automatisierte engpaorientierte Logistikanalyse
Copyright (C) 2001, 2002 Dirk Swiniartzki pamela\@planet-dirk.de

Dieses Programm ist freie Software. Sie knnen es unter
den Bedingungen der GNU General Public License Version 2,
wie von der Free Software Foundation herausgegeben, weitergeben
und/oder modifizieren.

Die Verffentlichung dieses Programms erfolgt in der
Hoffnung, da es Ihnen von Nutzen sein wird, aber OHNE JEDE
GEWHRLEISTUNG - sogar ohne die implizite Gewhrleistung
der MARKTREIFE oder der EIGNUNG FR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
ANNOUNCE

print "\nSyntax : pamela.plx <Konfigurationsdatei>\n\n";

@config=init(); # @config=($orderfile, $workplanfile, $resultdir, $startdate, $enddate, $refdate, $classes, $calendar, $intime)
$config[5]=convertdate($config[5]);

print "Daten (p)rfen oder (a)uswerten ? ";
$_=<STDIN>;
chomp;

if (/^p/) {
   @config=checkdata(@config);}
else {
   $config[3]=convertdate($config[3])-$config[5];
   $config[4]=convertdate($config[4])-$config[5];
   ($config[0], $min, $max)=calculate_ZAU(@config);
   calculate_DUDI($min, $max, @config);
   calculate_PKL(@config);
   calculate_TA(@config);}

__END__
